@@ -1,6 +1,6 @@
 // DasherInterfaceBase.cpp
 //
-// Copyright (c) 2007 The Dasher Team
+// Copyright (c) 2008 The Dasher Team
 //
 // This file is part of Dasher.
 //
@@ -38,7 +38,7 @@
 #include "BasicLog.h"
 #endif
 #include "WrapperFactory.h"
-//#include "DasherGameMode.h"
+#include "DasherGameMode.h"
 
 // Input filters
 #include "ClickFilter.h" 
@@ -51,14 +51,10 @@
 #include "TwoButtonDynamicFilter.h"
 
 // STL headers
+#include <cstdio>
 #include <iostream>
 #include <memory>
 
-// Legacy C library headers
-namespace {
-  #include "stdio.h"
-}
-
 // Declare our global file logging object
 #include "../DasherCore/FileLogger.h"
 #ifdef _DEBUG
@@ -106,7 +102,7 @@ CDasherInterfaceBase::CDasherInterfaceBase() {
   m_iCurrentState = ST_START;
   
   m_iLockCount = 0;
-  m_bGlobalLock = false;
+  //  m_bGlobalLock = false;
 
   // TODO: Are these actually needed?
   strCurrentContext = ". ";
@@ -117,6 +113,8 @@ CDasherInterfaceBase::CDasherInterfaceBase() {
 
   m_bLastChanged = true;
 
+  //  m_pTrainingHelper = new CTrainingHelper;
+
 #ifndef _WIN32_WCE
   // Global logging object we can use from anywhere
   g_pLogger = new CFileLogger("dasher.log",
@@ -124,6 +122,8 @@ CDasherInterfaceBase::CDasherInterfaceBase() {
                               g_iLogOptions);
 #endif
 
+  m_iNextLockID = 1;
+
 }
 
 void CDasherInterfaceBase::Realize() {
@@ -167,6 +167,13 @@ void CDasherInterfaceBase::Realize() {
   CreateInputFilter();
   SetupActionButtons();
 
+  // Set up real orientation to match selection
+  if(GetLongParameter(LP_ORIENTATION) == Dasher::Opts::AlphabetDefault)
+    SetLongParameter(LP_REAL_ORIENTATION, m_Alphabet->GetOrientation());
+  else
+    SetLongParameter(LP_REAL_ORIENTATION, GetLongParameter(LP_ORIENTATION));
+
+
   // FIXME - need to rationalise this sort of thing.
   // InvalidateContext(true);
   ScheduleRedraw();
@@ -181,8 +188,11 @@ void CDasherInterfaceBase::Realize() {
   // TODO: Make things work when model is created latet
   ChangeState(TR_MODEL_INIT);
 
+  using GameMode::CDasherGameMode;
   // Create the teacher singleton object. 
-  //  CDasherGameMode::CreateTeacher(m_pEventHandler, m_pSettingsStore, this);
+  CDasherGameMode::CreateTeacher(m_pEventHandler, m_pSettingsStore, this);
+  CDasherGameMode::GetTeacher()->SetDasherView(m_pDasherView);
+  CDasherGameMode::GetTeacher()->SetDasherModel(m_pDasherModel);
 }
 
 CDasherInterfaceBase::~CDasherInterfaceBase() {
@@ -190,7 +200,7 @@ CDasherInterfaceBase::~CDasherInterfaceBase() {
 
   // It may seem odd that InterfaceBase does not "own" the teacher.
   // This is because game mode is a different layer, in a sense.
-  //  CDasherGameMode::DestroyTeacher();
+  GameMode::CDasherGameMode::DestroyTeacher();
 
   delete m_pDasherModel;        // The order of some of these deletions matters
   delete m_Alphabet;
@@ -199,6 +209,7 @@ CDasherInterfaceBase::~CDasherInterfaceBase() {
   delete m_AlphIO;
   delete m_pInputFilter;
   delete m_pNCManager;
+  //  delete m_pTrainingHelper;
   // Do NOT delete Edit box or Screen. This class did not create them.
 
 #ifndef _WIN32_WCE
@@ -216,6 +227,13 @@ CDasherInterfaceBase::~CDasherInterfaceBase() {
   }
 #endif
 
+
+  for (std::vector<CActionButton *>::iterator it=m_vLeftButtons.begin(); it != m_vLeftButtons.end(); ++it)
+    delete *it;
+
+  for (std::vector<CActionButton *>::iterator it=m_vRightButtons.begin(); it != m_vRightButtons.end(); ++it)
+    delete *it;
+  
   // Must delete event handler after all CDasherComponent derived classes
 
   delete m_pEventHandler;
@@ -250,7 +268,7 @@ void CDasherInterfaceBase::PreSetNotify(int iParameter, const std::string &sNewV
 
 void CDasherInterfaceBase::InterfaceEventHandler(Dasher::CEvent *pEvent) {
 
-  if(pEvent->m_iEventType == 1) {
+  if(pEvent->m_iEventType == EV_PARAM_NOTIFY) {
     Dasher::CParameterNotificationEvent * pEvt(static_cast < Dasher::CParameterNotificationEvent * >(pEvent));
 
     switch (pEvt->m_iParameter) {
@@ -311,20 +329,20 @@ void CDasherInterfaceBase::InterfaceEventHandler(Dasher::CEvent *pEvent) {
       break;
     }
   }
-  else if(pEvent->m_iEventType == 2) {
+  else if(pEvent->m_iEventType == EV_EDIT && !GetBoolParameter(BP_GAME_MODE)) {
     CEditEvent *pEditEvent(static_cast < CEditEvent * >(pEvent));
     
     if(pEditEvent->m_iEditType == 1) {
       strCurrentContext += pEditEvent->m_sText;
       if( strCurrentContext.size() > 20 )
 	strCurrentContext = strCurrentContext.substr( strCurrentContext.size() - 20 );
-
-      strTrainfileBuffer += pEditEvent->m_sText;
+      if(GetBoolParameter(BP_LM_ADAPTIVE))
+	 strTrainfileBuffer += pEditEvent->m_sText;
     }
     else if(pEditEvent->m_iEditType == 2) {
       strCurrentContext = strCurrentContext.substr( 0, strCurrentContext.size() - pEditEvent->m_sText.size());
-
-      strTrainfileBuffer = strTrainfileBuffer.substr( 0, strTrainfileBuffer.size() - pEditEvent->m_sText.size());
+      if(GetBoolParameter(BP_LM_ADAPTIVE))
+	 strTrainfileBuffer = strTrainfileBuffer.substr( 0, strTrainfileBuffer.size() - pEditEvent->m_sText.size());
     }
   }
   else if(pEvent->m_iEventType == EV_CONTROL) {
@@ -340,7 +358,7 @@ void CDasherInterfaceBase::InterfaceEventHandler(Dasher::CEvent *pEvent) {
     }
   }
   else if(pEvent->m_iEventType == EV_LOCK) {
-    CLockEvent *pLockEvent(static_cast<CLockEvent *>(pEvent));
+//    CLockEvent *pLockEvent(static_cast<CLockEvent *>(pEvent));
 
 //     // TODO: Sort this out - at the moment these don't occur in pairs, so the old boolean variable is still needed
 //     if(pLockEvent->m_bLock) {
@@ -352,13 +370,12 @@ void CDasherInterfaceBase::InterfaceEventHandler(Dasher::CEvent *pEvent) {
 // 	ReleaseLock(0);
 //     }
     
-    m_bGlobalLock = pLockEvent->m_bLock;
+//    m_bGlobalLock = pLockEvent->m_bLock;
   }
 }
 
 void CDasherInterfaceBase::WriteTrainFileFull() {
-  if(!GetBoolParameter(BP_GAME_MODE))
-    WriteTrainFile(strTrainfileBuffer);
+  WriteTrainFile(strTrainfileBuffer);
   strTrainfileBuffer = "";
 }
 
@@ -381,8 +398,8 @@ void CDasherInterfaceBase::CreateModel(int iOffset) {
   m_pDasherModel = new CDasherModel(m_pEventHandler, m_pSettingsStore, m_pNCManager, this, m_pDasherView, iOffset);
   
   // Notify the teacher of the new model
-  //  if(CDasherGameMode* pTeacher = CDasherGameMode::GetTeacher())
-  //  pTeacher->SetDasherModel(m_pDasherModel);
+  if(GameMode::CDasherGameMode* pTeacher = GameMode::CDasherGameMode::GetTeacher())
+    pTeacher->SetDasherModel(m_pDasherModel);
   
 }
 
@@ -398,13 +415,14 @@ void CDasherInterfaceBase::CreateNCManager() {
     return;
 
  // Train the new language model
-    CLockEvent *pEvent;
+  //    CLockEvent *pEvent;
     
-    pEvent = new CLockEvent("Training Dasher", true, 0);
-    m_pEventHandler->InsertEvent(pEvent);
-    delete pEvent;
+    //    pEvent = new CLockEvent("Training Dasher", true, 0);
+    //  m_pEventHandler->InsertEvent(pEvent);
+    //  delete pEvent;
 
-    AddLock(0);
+    // TODO: Move training into nc manager
+    int iTrainingLock = AddLock("Training Dasher");
 
     int iOffset;
 
@@ -427,29 +445,15 @@ void CDasherInterfaceBase::CreateNCManager() {
     m_pNCManager = new CNodeCreationManager(this, m_pEventHandler, m_pSettingsStore, m_AlphIO);
 
     m_Alphabet = m_pNCManager->GetAlphabet();
-    
-    // TODO: Consider using GetStringParameter(SP_TRAIN_FILE) instead?
-    string T = m_Alphabet->GetTrainingFile();
+   
+    ReleaseLock(iTrainingLock);
 
-    int iTotalBytes = 0;
-    iTotalBytes += GetFileSize(GetStringParameter(SP_SYSTEM_LOC) + T);
-    iTotalBytes += GetFileSize(GetStringParameter(SP_USER_LOC) + T);
 
-    if(iTotalBytes > 0) {
-      int iOffset;
-      iOffset = TrainFile(GetStringParameter(SP_SYSTEM_LOC) + T, iTotalBytes, 0);
-      TrainFile(GetStringParameter(SP_USER_LOC) + T, iTotalBytes, iOffset);
-    }
-    else {
-      CMessageEvent oEvent("No training text is avilable for the selected alphabet. Dasher will function, but it may be difficult to enter text.\nPlease see http://www.dasher.org.uk/alphabets/ for more information.", 0, 0);
-      m_pEventHandler->InsertEvent(&oEvent);
-    }
+//     pEvent = new CLockEvent("Training Dasher", false, 0);
+//     m_pEventHandler->InsertEvent(pEvent);
+//     delete pEvent;
 
-    pEvent = new CLockEvent("Training Dasher", false, 0);
-    m_pEventHandler->InsertEvent(pEvent);
-    delete pEvent;
-
-    ReleaseLock(0);
+//     ReleaseLock(0);
 
     // TODO: Eventually we'll not have to pass the NC manager to the model...
     CreateModel(iOffset);
@@ -471,7 +475,7 @@ void CDasherInterfaceBase::PauseAt(int MouseX, int MouseY) {
 }
 
 void CDasherInterfaceBase::GameMessageIn(int message, void* messagedata) {
-  //  CDasherGameMode::GetTeacher()->Message(message, messagedata);
+  GameMode::CDasherGameMode::GetTeacher()->Message(message, messagedata);
 }
 
 
@@ -492,7 +496,9 @@ void CDasherInterfaceBase::Unpause(unsigned long Time) {
   Dasher::CStartEvent oEvent;
   m_pEventHandler->InsertEvent(&oEvent);
 
-  ResetNats();
+  // Commenting this out, can't see a good reason to ResetNats,
+  // just because we are not paused anymore - pconlon
+  // ResetNats();
 
 #ifndef _WIN32_WCE
   if (m_pUserLog != NULL)
@@ -518,9 +524,15 @@ void CDasherInterfaceBase::CreateInput() {
 }
 
 void CDasherInterfaceBase::NewFrame(unsigned long iTime, bool bForceRedraw) {
+  // Prevent NewFrame from being reentered. This can happen occasionally and
+  // cause crashes. 
+  static bool bReentered=false;
+  if(bReentered) return;
+  bReentered=true;
+  
   // Fail if Dasher is locked
-  if(m_iCurrentState != ST_NORMAL)
-    return;
+  // if(m_iCurrentState != ST_NORMAL)
+  //  return;
 
   bool bChanged(false);
 
@@ -576,7 +588,8 @@ void CDasherInterfaceBase::NewFrame(unsigned long iTime, bool bForceRedraw) {
   if(m_pDasherModel != 0)
     m_pDasherModel->NewFrame(iTime);
 
- }
+  bReentered=false;
+}
 
 void CDasherInterfaceBase::CheckRedraw() {
   if(m_bRedrawScheduled)
@@ -585,25 +598,22 @@ void CDasherInterfaceBase::CheckRedraw() {
   m_bRedrawScheduled = false;
 }
 
-/// Full redraw - renders model, decorations and blits onto display
-/// buffer. Not recommended if nothing has changed with the model,
-/// otherwise we're wasting effort.
-
 void CDasherInterfaceBase::Redraw(bool bRedrawNodes) {
-  if(!m_pDasherView || !m_pDasherModel)
+  // No point continuing if there's nothing to draw on...
+  if(!m_pDasherView)
     return;
   
-  // TODO: More generic notion of 'layers'?
-
+  // Draw the nodes
   if(bRedrawNodes) {
     m_pDasherView->Screen()->SendMarker(0);
-    m_pDasherModel->RenderToView(m_pDasherView, true);
+    m_pDasherView->RenderModel(m_pDasherModel);
   }
-  
+
+  // Draw the decorations
   m_pDasherView->Screen()->SendMarker(1);
   
-  //  if(CDasherGameMode* pTeacher = CDasherGameMode::GetTeacher())
-  // pTeacher->DrawGameDecorations(m_pDasherView);
+  if(GameMode::CDasherGameMode* pTeacher = GameMode::CDasherGameMode::GetTeacher())
+    pTeacher->DrawGameDecorations(m_pDasherView);
     
   bool bDecorationsChanged(false);
 
@@ -616,6 +626,12 @@ void CDasherInterfaceBase::Redraw(bool bRedrawNodes) {
   bActionButtonsChanged = DrawActionButtons();
 #endif
 
+  SLockData *pCurrentLock = GetCurrentLock();
+  if(pCurrentLock) {
+    std::cout << "Rendering lock" << std::endl;
+  }
+
+  // Only blit the image to the display if something has actually changed
   if(bRedrawNodes || bDecorationsChanged || bActionButtonsChanged)
     m_pDasherView->Display();
 }
@@ -677,19 +693,19 @@ void CDasherInterfaceBase::ChangeScreen(CDasherScreen *NewScreen) {
 
 void CDasherInterfaceBase::ChangeView() {
   // TODO: Actually respond to LP_VIEW_ID parameter (although there is only one view at the moment)
-
-  if(m_DasherScreen != 0 && m_pDasherModel != 0) {
+  
+  // removed condition that m_pDasherModel != 0. Surely the view can exist without the model?-pconlon
+    if(m_DasherScreen != 0 /*&& m_pDasherModel != 0*/) { 
     delete m_pDasherView;
     
     m_pDasherView = new CDasherViewSquare(m_pEventHandler, m_pSettingsStore, m_DasherScreen);
  
     if (m_pInput)
       m_pDasherView->SetInput(m_pInput);
+
     // Tell the Teacher which view we are using
-    //    if(CDasherGameMode* pTeacher = CDasherGameMode::GetTeacher())
-    // {/
-    //	pTeacher->SetDasherView(m_pDasherView);
-    // }
+    if(GameMode::CDasherGameMode* pTeacher = GameMode::CDasherGameMode::GetTeacher())
+	pTeacher->SetDasherView(m_pDasherView);
   }
 }
 
@@ -705,74 +721,6 @@ void CDasherInterfaceBase::DeleteAlphabet(const std::string &AlphID) {
   m_AlphIO->Delete(AlphID);
 }
 
-/*
-	I've used C style I/O because I found that C++ style I/O bloated
-	the Win32 code enormously. The overhead of loading the buffer into
-	the string instead of reading straight into a string seems to be
-	negligible compared to huge requirements elsewhere.
-*/
-int CDasherInterfaceBase::TrainFile(std::string Filename, int iTotalBytes, int iOffset) {
-  if(Filename == "")
-    return 0;
-  
-  FILE *InputFile;
-  if((InputFile = fopen(Filename.c_str(), "r")) == (FILE *) 0)
-    return 0;
-
-  AddLock(0);
-
-  const int BufferSize = 1024;
-  char InputBuffer[BufferSize];
-  string StringBuffer;
-  int NumberRead;
-  int iTotalRead(0);
-
-  vector < symbol > Symbols;
-
-  CAlphabetManagerFactory::CTrainer * pTrainer = m_pNCManager->GetTrainer();
-  do {
-    NumberRead = fread(InputBuffer, 1, BufferSize - 1, InputFile);
-    InputBuffer[NumberRead] = '\0';
-    StringBuffer += InputBuffer;
-    bool bIsMore = false;
-    if(NumberRead == (BufferSize - 1))
-      bIsMore = true;
-
-    Symbols.clear();
-    m_Alphabet->GetSymbols(&Symbols, &StringBuffer, bIsMore);
-
-    pTrainer->Train(Symbols);
-    iTotalRead += NumberRead;
-  
-    // TODO: No reason for this to be a pointer (other than cut/paste laziness!)
-    CLockEvent *pEvent;
-    pEvent = new CLockEvent("Training Dasher", true, static_cast<int>((100.0 * (iTotalRead + iOffset))/iTotalBytes));
-    m_pEventHandler->InsertEvent(pEvent);
-    delete pEvent;
-
-  } while(NumberRead == BufferSize - 1);
-
-  delete pTrainer;
-
-  fclose(InputFile);
-
-  ReleaseLock(0);
-
-  return iTotalRead;
-}
-
-void CDasherInterfaceBase::GetFontSizes(std::vector <int >*FontSizes) const {
-  FontSizes->push_back(20);
-  FontSizes->push_back(14);
-  FontSizes->push_back(11);
-  FontSizes->push_back(40);
-  FontSizes->push_back(28);
-  FontSizes->push_back(22);
-  FontSizes->push_back(80);
-  FontSizes->push_back(56);
-  FontSizes->push_back(44);
-}
-
 double CDasherInterfaceBase::GetCurCPM() {
   //
   return 0;
@@ -859,7 +807,7 @@ void CDasherInterfaceBase::ResetNats() {
 
 std::string CDasherInterfaceBase::GetContext(int iStart, int iLength) {
   m_strContext = "";
-  
+
   CEditContextEvent oEvent(iStart, iLength);
   m_pEventHandler->InsertEvent(&oEvent);
 
@@ -1182,21 +1130,46 @@ void CDasherInterfaceBase::EnterState(EState iState) {
   }
 }
 
-void CDasherInterfaceBase::AddLock(int iLockFlags) {
-  if(m_iLockCount == 0)
+int CDasherInterfaceBase::AddLock(const std::string &strDisplay) {
+  std::cout << "Adding lock " << strDisplay << std::endl;
+  SLockData sNewLock;
+
+  sNewLock.strDisplay = strDisplay;
+  sNewLock.iPercent = 0;
+
+  if(m_mapCurrentLocks.size() == 0)
     ChangeState(TR_LOCK);
 
-  ++m_iLockCount;
+  m_mapCurrentLocks[m_iNextLockID] = sNewLock;
+  ++m_iNextLockID;
+
+  Redraw(false);
+
+  return (m_iNextLockID - 1);
 }
 
-void CDasherInterfaceBase::ReleaseLock(int iLockFlags) {
-  if(m_iLockCount > 0)
-    --m_iLockCount;
+void CDasherInterfaceBase::ReleaseLock(int iLockID) {
+  std::cout << "Releasing Lock" << std::endl;
+  std::map<int, SLockData>::iterator it = m_mapCurrentLocks.find(iLockID);
 
-  if(m_iLockCount == 0)
+  if(it != m_mapCurrentLocks.end()) {
+    m_mapCurrentLocks.erase(it);
+  }
+
+  Redraw(false);
+
+  if(m_mapCurrentLocks.size() == 0)
     ChangeState(TR_UNLOCK);
 }
 
+SLockData *CDasherInterfaceBase::GetCurrentLock() {
+
+  if(m_mapCurrentLocks.size() > 0)
+    return &(m_mapCurrentLocks.begin()->second);
+  else
+    return NULL;
+}
+
 void CDasherInterfaceBase::SetBuffer(int iOffset) { 
    CreateModel(iOffset);
 }
@@ -1223,3 +1196,11 @@ void CDasherInterfaceBase::ClSet(const std::string &strKey, const std::string &s
   if(m_pSettingsStore)
     m_pSettingsStore->ClSet(strKey, strValue);
 }
+
+
+void 
+CDasherInterfaceBase::ImportTrainingText(const std::string &strPath) {
+  if(m_pNCManager)
+    m_pNCManager->ImportTrainingText(strPath);
+}
+
