@@ -31,6 +31,15 @@ LRESULT CStatusControl::OnNotify(UINT message, WPARAM wParam, LPARAM lParam, BOO
     case UDN_DELTAPOS:
       UpdateSpeed(((LPNMUPDOWN) lParam)->iPos, ((LPNMUPDOWN) lParam)->iDelta);
       break;
+    case EN_CHANGE:
+      {
+        TCHAR wszBuffer[32];
+        SendMessage(m_hEdit, WM_GETTEXT, 32, (long)wszBuffer);
+        double dNewSpeed = _tstof(wszBuffer);
+
+        m_pDasherInterface->SetLongParameter(LP_MAX_BITRATE, dNewSpeed * 100);
+      }
+      break;
     default:
       bHandled = false;
       break;
@@ -45,7 +54,7 @@ LRESULT CStatusControl::OnSize(UINT message, WPARAM wParam, LPARAM lParam, BOOL&
 }
 
 HWND CStatusControl::Create(HWND hParent) {
-  CWindowImpl<CStatusControl>::Create(hParent, 0, 0, SS_WHITERECT | WS_VISIBLE | WS_CHILD );
+  CWindowImpl<CStatusControl>::Create(hParent, 0, 0, WS_VISIBLE | WS_CHILD );
 
   CreateChildren();
 
@@ -66,11 +75,22 @@ void CStatusControl::CreateChildren() {
   std::wstring strAlphabetLabel(L"Alphabet:");
 
   // TODO: Wrap windows here in CWindow classes.
+#ifndef _WIN32_WCE
+  // TODO: Is this really needed?
   m_hSpeedLabel = CreateWindowEx(WS_EX_CONTROLPARENT, TEXT("STATIC"), strSpeedLabel.c_str(), 
-      SS_CENTER | SS_WHITERECT | SS_WHITEFRAME | WS_CHILD | WS_VISIBLE | WS_TABSTOP, 0, 0, 0, 0, m_hWnd, NULL, WinHelper::hInstApp, NULL);
+      SS_CENTER | WS_CHILD | WS_VISIBLE | WS_TABSTOP, 0, 0, 0, 0, m_hWnd, NULL, WinHelper::hInstApp, NULL);
+#else
+  m_hSpeedLabel = CreateWindowEx(0, TEXT("STATIC"), strSpeedLabel.c_str(), 
+      SS_CENTER | WS_CHILD | WS_VISIBLE | WS_TABSTOP, 0, 0, 0, 0, m_hWnd, NULL, WinHelper::hInstApp, NULL);
+#endif
 
+#ifndef _WIN32_WCE
   m_hAlphabetLabel = CreateWindowEx(WS_EX_CONTROLPARENT, TEXT("STATIC"), strAlphabetLabel.c_str(), 
-      SS_CENTER | SS_WHITERECT | SS_WHITEFRAME | WS_CHILD | WS_VISIBLE | WS_TABSTOP, 0, 0, 0, 0, m_hWnd, NULL, WinHelper::hInstApp, NULL);
+      SS_CENTER | WS_CHILD | WS_VISIBLE | WS_TABSTOP, 0, 0, 0, 0, m_hWnd, NULL, WinHelper::hInstApp, NULL);
+#else
+  m_hAlphabetLabel = CreateWindowEx(0, TEXT("STATIC"), strAlphabetLabel.c_str(), 
+      SS_CENTER | WS_CHILD | WS_VISIBLE | WS_TABSTOP, 0, 0, 0, 0, m_hWnd, NULL, WinHelper::hInstApp, NULL);
+#endif
 
   SendMessage(m_hSpeedLabel, WM_SETFONT, (WPARAM) hGuiFont, true);
   SendMessage(m_hAlphabetLabel, WM_SETFONT, (WPARAM) hGuiFont, true);
@@ -113,7 +133,7 @@ void CStatusControl::CreateChildren() {
   DeleteObject(hGuiFont);
 
   // Set the height to what is finally required. The extra pixel is needed to get everything to line up nicely
-  MoveWindow(0, 0, 0, m_iEditHeight + 1, false);
+  MoveWindow(0, 0, 0, m_iEditHeight+1, false);
 }
 
 void CStatusControl::LayoutChildrenInitial() {
@@ -200,7 +220,7 @@ void CStatusControl::SelectAlphabet() {
   SendMessage(m_hCombo, CB_GETLBTEXT, iIndex, (LPARAM)szSelection);
 
   if(!_tcscmp(szSelection, L"More Alphabets...")) {
-    SendMessage(::GetParent(GetParent().m_hWnd), DASHER_SHOW_PREFS, 0, 0);
+    SendMessage(GetParent().m_hWnd, DASHER_SHOW_PREFS, 0, 0);
   }
   else {
     std::string strNewValue;
